package ru.dbotthepony.kstarbound.test

import org.junit.jupiter.api.DisplayName
import org.junit.jupiter.api.Test
import ru.dbotthepony.kstarbound.math.*
import ru.dbotthepony.kstarbound.world.api.CHUNK_SIZE
import ru.dbotthepony.kstarbound.world.api.CHUNK_SIZE_FF
import ru.dbotthepony.kstarbound.world.ChunkPos

object MathTests {
	@Test
	@DisplayName("roundByAbsoluteValue test")
	fun roundByAbsoluteValueTest() {
		check(roundByAbsoluteValue(0.0) == 0)
		check(roundByAbsoluteValue(0.1) == 1)
		check(roundByAbsoluteValue(1.1) == 2)
		check(roundByAbsoluteValue(-0.1) == -1)
		check(roundByAbsoluteValue(-0.0) == 0)
		check(roundByAbsoluteValue(-1.0) == -1)
		check(roundByAbsoluteValue(-1.1) == -2)
	}

	@Test
	@DisplayName("roundTowardsNegativeInfinity test")
	fun roundTowardsNegativeInfinityTest() {
		check(roundTowardsNegativeInfinity(0.0) == 0)
		check(roundTowardsNegativeInfinity(0.1) == 0)
		check(roundTowardsNegativeInfinity(0.6) == 0)
		check(roundTowardsNegativeInfinity(1.1) == 1)
		check(roundTowardsNegativeInfinity(-0.1) == -1)
		check(roundTowardsNegativeInfinity(-0.0) == 0)
		check(roundTowardsNegativeInfinity(-1.0) == -1)
		check(roundTowardsNegativeInfinity(-1.1) == -2)
		check(roundTowardsNegativeInfinity(-1.6) == -2)
	}

	@Test
	@DisplayName("roundTowardsPositiveInfinity test")
	fun roundTowardsPositiveInfinityTest() {
		check(roundTowardsPositiveInfinity(0.0) == 0)
		check(roundTowardsPositiveInfinity(0.1) == 1)
		check(roundTowardsPositiveInfinity(0.6) == 1)
		check(roundTowardsPositiveInfinity(1.1) == 2)
		check(roundTowardsPositiveInfinity(-0.1) == 0)
		check(roundTowardsPositiveInfinity(-0.0) == 0)
		check(roundTowardsPositiveInfinity(-1.0) == -1)
		check(roundTowardsPositiveInfinity(-1.1) == -1)
		check(roundTowardsPositiveInfinity(-1.6) == -1)
	}

	@Test
	@DisplayName("ChunkPos class tests")
	fun chunkPosTests() {
		check(ChunkPos.fromPosition(0, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(1, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(0, 1) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(1, 1) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(-1, 1) == ChunkPos(-1, 0))
		check(ChunkPos.fromPosition(-1, -1) == ChunkPos(-1, -1))

		check(ChunkPos.fromPosition(CHUNK_SIZE_FF, 0) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(CHUNK_SIZE, 0) == ChunkPos(1, 0))
		check(ChunkPos.fromPosition(0, CHUNK_SIZE_FF) == ChunkPos(0, 0))
		check(ChunkPos.fromPosition(0, CHUNK_SIZE) == ChunkPos(0, 1))

		check(ChunkPos.fromPosition(-CHUNK_SIZE_FF, 0) == ChunkPos(-1, 0)) { ChunkPos.fromPosition(-CHUNK_SIZE_FF, 0).toString() }
		check(ChunkPos.fromPosition(-CHUNK_SIZE, 0) == ChunkPos(-2, 0)) { ChunkPos.fromPosition(-CHUNK_SIZE, 0) }
		check(ChunkPos.fromPosition(0, -CHUNK_SIZE_FF) == ChunkPos(0, -1)) { ChunkPos.fromPosition(0, -CHUNK_SIZE_FF) }
		check(ChunkPos.fromPosition(0, -CHUNK_SIZE) == ChunkPos(0, -2)) { ChunkPos.fromPosition(0, -CHUNK_SIZE) }
	}
}
